<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_preloaded_regular_screen.php';

///////////////////////////////////////////////////////////////////////////

class TvChannelListScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'tv_channel_list';

    public static function get_media_url_str($group_id)
    {
        return MediaURL::encode(
            array
            (
                'screen_id' => self::ID,
                'group_id'  => $group_id,
            ));
    }

    ///////////////////////////////////////////////////////////////////////

    protected $tv;
	private $select_id = false;
    ///////////////////////////////////////////////////////////////////////

    public function __construct(Tv $tv, $folder_views)
    {
        parent::__construct(self::ID, $folder_views);
        $this->tv = $tv;
        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        $actions = array();
		$actions[GUI_EVENT_KEY_SETUP] = ActionFactory::open_folder(SmartSetupScreen::get_media_url_str(),'%tr%t196');
		$add_action = UserInputHandlerRegistry::create_action($this, 'add_id');
		$add_action['caption'] = '%tr%t227';
		$actions[GUI_EVENT_KEY_B_GREEN] = $add_action;
		$add_action = UserInputHandlerRegistry::create_action($this, 'add_logo');
		$add_action['caption'] = '%tr%t222';
		$actions[GUI_EVENT_KEY_C_YELLOW] = $add_action;
		$actions[GUI_EVENT_KEY_SELECT] = UserInputHandlerRegistry::create_action($this, 'select');
		$actions[GUI_EVENT_KEY_CLEAR] = UserInputHandlerRegistry::create_action($this, 'hide_ch_apply');
		$add_action = UserInputHandlerRegistry::create_action($this, 'search_channel');
		$actions[GUI_EVENT_KEY_SEARCH] = $add_action;
		$add_action = UserInputHandlerRegistry::create_action($this, 'up_channel');
		$actions[GUI_EVENT_KEY_FWD] = $add_action;
		$add_action = UserInputHandlerRegistry::create_action($this, 'lo_channel');
		$actions[GUI_EVENT_KEY_REW] = $add_action;
		$add_action = UserInputHandlerRegistry::create_action($this, 'sort_channels');
		$actions[GUI_EVENT_KEY_AUDIO] = $add_action;
		$add_action = UserInputHandlerRegistry::create_action($this, 'add_favorite');
		$add_action['caption'] = '%tr%t62';
		$actions[GUI_EVENT_KEY_D_BLUE] = $add_action;
        $actions[GUI_EVENT_KEY_POPUP_MENU] = UserInputHandlerRegistry::create_action($this, 'popup_menu');
        $actions[GUI_EVENT_KEY_INFO] = UserInputHandlerRegistry::create_action($this, 'info');
		$actions[GUI_EVENT_KEY_ENTER] = ActionFactory::tv_play();
        $actions[GUI_EVENT_KEY_PLAY] = ActionFactory::tv_play();
        return $actions;
    }

    public function get_handler_id()
    { return self::ID; }

    private function get_sel_item_update_action(&$user_input, &$plugin_cookies)
    {
        $parent_media_url = MediaURL::decode($user_input->parent_media_url);
        $sel_ndx = $user_input->sel_ndx;
        $group = $this->tv->get_group($parent_media_url->group_id);
        $channels = $group->get_channels($plugin_cookies);
        $items[] = $this->get_regular_folder_item($group,
            $channels->get_by_ndx($sel_ndx), $plugin_cookies);
        $range = HD::create_regular_folder_range($items,
            $sel_ndx, $channels->size());
        return ActionFactory::update_regular_folder($range, false);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        $sort_channels = isset($plugin_cookies->sort_channels) ? $plugin_cookies->sort_channels : 0;
		$attrs['dialog_params'] = array('frame_style' => DIALOG_FRAME_STYLE_GLASS);
		if ($user_input->control_id == 'info')
        {
            if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
            $channel_id = $media_url->channel_id;
            $channels = $this->tv->get_channels();
            $c = $channels->get($channel_id);
            $id = $c->get_id();
            $title = $c->get_title();
			$inf = $this->tv->global_channels[$channel_id];
			$defs = array();
			$line = '';
			if (preg_match("|[a-zA-Z]|", $id))
				ControlFactory::add_label($defs,'', '%tr%t228');
			else
				ControlFactory::add_label($defs,'', "vsetvID: $id");
			foreach ($inf['caption'] as $k => $v)
				$line .= "---------------------------------------------------------------------------------\nName:$v\nPlaylist:". $inf['urls'][$k] . "\nStream:" . $k ."\n";
			ControlFactory::add_multiline_label($defs, "", $line, 15);
            return ActionFactory::show_dialog("Channel '$title'",$defs,true,1300,$attrs);
        }
		
		if ($user_input->control_id == 'add_channel_list_bg')
        {
            if (!isset($user_input->selected_media_url))
                return null;
			$msg = '%tr%t44';
			$media_url = MediaURL::encode (
                    array
                    (
                        'screen_id'     => 'file_list',
						'save_file'		=> array(	
						'parent' => self::ID,
						'action' => 'plugin_ico',
						'arg' => 'channel_list_bg',
						'msg' => $msg,
						'extension' => 'jpg'
						),
                    )
			);
			return ActionFactory::open_folder($media_url,'%tr%t218');
		}
		
		if ($user_input->control_id == 'del_channel_list_bg')
        {
            if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$plugin_ico = HD::get_items('plugin_ico');
			unlink($plugin_ico['channel_list_bg']);
			unset ($plugin_ico['channel_list_bg']);
			HD::save_items('plugin_ico', $plugin_ico);
			return ActionFactory::invalidate_folders(array('vod_category_list'), 
			ActionFactory::show_title_dialog('%tr%t219',null,'%tr%t44',800));
		}
		
		if ($user_input->control_id == 'del_rename')
        {
            if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$user_list = array_flip (HD::get_items('vsetv_list'));
			unset($user_list[$channel_id]);
			$user_list = array_flip ($user_list);
			HD::save_items('vsetv_list', $user_list);
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id == 'rename')
        {
            if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			$c = $channels->get($channel_id);
			$title = $c->get_title();
			if (!isset($user_input->selected_media_url))
                return null;
			$defs = array();
			ControlFactory::add_text_field($defs,
				$this, $add_params=null,
				'newname', '',
				$title, 0, 0, 1, 1, 1300, 0, 1);
			ControlFactory::add_vgap($defs, 500);	
			return  ActionFactory::show_dialog('%tr%t229',$defs,true,0,$attrs);
		}
		if ($user_input->control_id == 'newname')
        {
			if ($user_input->newname == '')
				return ActionFactory::show_title_dialog('%tr%t200');
			$do = UserInputHandlerRegistry::create_action($this, 'add_newname_apply');
			return ActionFactory::close_dialog_and_run($do);
		}
		if ($user_input->control_id == 'add_newname_apply')
        {
            if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$user_list = array_flip (HD::get_items('vsetv_list'));
			$user_list[$channel_id] = $user_input->newname;
			$user_list = array_flip ($user_list);
			HD::save_items('vsetv_list', $user_list);
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		if ($user_input->control_id == 'select')
        {
            if (!isset($user_input->selected_media_url))
                return null;
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$links = $this->select_id;
			if (isset($links[$media_url->channel_id]))
				unset ($links[$media_url->channel_id]);
			else
				$links[$media_url->channel_id] = 1;
			$this->select_id = $links;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder_plus');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		if ($user_input->control_id == 'search_channel'){
			$defs = array();
			$media_url = MediaURL::decode($user_input->selected_media_url);
            $channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			$c = $channels->get($channel_id);
			$title = $c->get_title();
			$use_osk = (HD::get_item('use_osk') !='') ? HD::get_item('use_osk') : 1;
			ControlFactory::add_text_field(
				$defs, $this, null,
				'do_new_search', '',
				$title, 0, 0, $use_osk, 1, 1300,0,1
			);
			if ($use_osk == 1)
				ControlFactory::add_vgap($defs, 500);
			return  ActionFactory::show_dialog('%tr%t230', $defs,true,1300,$attrs);
		}
		
		if ($user_input->control_id == 'do_new_search')
        {
			$do_search = UserInputHandlerRegistry::create_action($this, 'run_search');
			return ActionFactory::close_dialog_and_run($do_search);
		}
		
		if ($user_input->control_id == 'run_search')
		{
			$defs = array();
			$find_text = $user_input->do_new_search;
			$playlist = $this->tv->global_channels;
			$categories = SmartConfig::get_group();
			$categories["__all_channels"] = '%tr%t63';
			$hide_ch_defs = HD::get_items('hide_ch');
			$group_defs = HD::get_items('grups_id');
			$i = 0;
			$q = false;
			foreach ($playlist as $id => $tv_channel){
				$h = false;
				foreach ($tv_channel['caption'] as $caption)
					$s = mb_stripos($caption, $find_text, 0, "UTF-8");
				if (array_key_exists($id, $hide_ch_defs)){
					$caption = $caption . '(hide)';
					$h = true;
				}
				if ($s !== false){
					$q = true;
					$group_id = (array_key_exists($id, $group_defs)) ? $group_defs[$id] : "__all_channels";
					$group_name = $categories[strval($group_id)];
					if ($sort_channels == 0)
						$add_params['number'] = intval($i);
					if ($sort_channels == 1){
						$chnmbr_list = array_flip ($this->tv->nbr_channels);
						$add_params['number'] = intval($chnmbr_list[$id]);
					}
					if (($sort_channels == 2)||($sort_channels == 3)){
						$chnmbr_list = array_flip (HD::get_items('chnmbr_list'));
						$add_params['number'] = intval($chnmbr_list[$id]);
					}						
					ControlFactory::add_close_dialog_and_apply_button_title(&$defs, $this, $add_params, 'jump', strval($group_name), strval($caption), 900, false);
				}
				if ($h == false)
					$i++;
			}
			if ($q == false){
				ControlFactory::add_multiline_label(&$defs, '', '%tr%t231',6);
				ControlFactory::add_vgap($defs, 20);
				ControlFactory::add_close_dialog_and_apply_button_title($defs, $this, null, 'search_channel', '', '%tr%t232', 300, true);
			}
			return  ActionFactory::show_dialog('%tr%t232', $defs, true, 0, $attrs);
		}
		
		if ($user_input->control_id == 'jump')
		{
			$ndx = $user_input->number;
			$parent_media_url = MediaURL::decode($user_input->parent_media_url);
			$parent_media_url->group_id = "__all_channels";
			$range = $this->get_folder_range($parent_media_url, 0, $plugin_cookies);
			return ActionFactory::update_regular_folder($range, true, $ndx);
		}
		if ($user_input->control_id === 'sort_channels'){
			$n =  isset($plugin_cookies->sort_channels) ? $plugin_cookies->sort_channels : 0;
			if ($n == 3)
				$n = 0;
			else
				$n = $n + 1;
			$show_sort[0] = '%tr%t154';
			$show_sort[1] = '%tr%t155';
			$show_sort[2] = '%tr%t156';
			$show_sort[3] = '%tr%t157';
			$plugin_cookies->sort_channels = $n;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::show_title_dialog($show_sort[$n],
			ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action),0,0,1);
		}
		if ($user_input->control_id === 'up_channel'){
			if (($sort_channels == 0)||($sort_channels == 1))
				return ActionFactory::show_title_dialog('err!!!',	null,'%tr%t234',950,1);
			$chnmbr_list = HD::get_items('chnmbr_list');
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$k = array_search($channel_id, $chnmbr_list);
            if ($k !== false && $k !== 0)
            {
                $t = $chnmbr_list[$k - 1];
                $chnmbr_list[$k - 1] = $chnmbr_list[$k];
                $chnmbr_list[$k] = $t;
            }
			$chnmbr_list = array_values($chnmbr_list);
			HD::save_items('chnmbr_list', $chnmbr_list);
			$params['cursor'] = -1;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder', $params);
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		if ($user_input->control_id === 'lo_channel'){
			if (($sort_channels == 0)||($sort_channels == 1))
				return ActionFactory::show_title_dialog('err!!!',	null,'%tr%t234',950,1);
			$chnmbr_list = HD::get_items('chnmbr_list');
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$k = array_search($channel_id, $chnmbr_list);
            if ($k !== false && $k !== count($chnmbr_list) - 1)
            {
                $t = $chnmbr_list[$k + 1];
                $chnmbr_list[$k + 1] = $chnmbr_list[$k];
                $chnmbr_list[$k] = $t;
            }
			$chnmbr_list = array_values($chnmbr_list);
			HD::save_items('chnmbr_list', $chnmbr_list);
			$params['cursor'] = 1;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder', $params);
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		if ($user_input->control_id === 'cleer_chn'){
			$chnmbr_list = array();
			HD::save_items('chnmbr_list', $chnmbr_list);
			$params['cursor'] = 0;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder', $params);
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		if ($user_input->control_id === 'first_channel'){
			
			$chnmbr_list = HD::get_items('chnmbr_list');
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$flip_chnmbr_list = array_flip ($chnmbr_list);
			if (isset($flip_chnmbr_list[$media_url->channel_id])){
				unset ($flip_chnmbr_list[$media_url->channel_id]);
				$chnmbr_list = array_flip ($flip_chnmbr_list);
			}
			array_unshift($chnmbr_list, $media_url->channel_id);
			HD::save_items('chnmbr_list', $chnmbr_list);
			
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		if ($user_input->control_id === 'end_channel'){
			
			$chnmbr_list = HD::get_items('chnmbr_list');
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$flip_chnmbr_list = array_flip ($chnmbr_list);
			if (isset($flip_chnmbr_list[$media_url->channel_id])){
				unset ($flip_chnmbr_list[$media_url->channel_id]);
				$chnmbr_list = array_flip ($flip_chnmbr_list);
			}
			array_push($chnmbr_list, $media_url->channel_id);
			HD::save_items('chnmbr_list', $chnmbr_list);
			
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
        if ($user_input->control_id == 'popup_menu')
        {
            if (!isset($user_input->selected_media_url))
                return null;

            $media_url = MediaURL::decode($user_input->selected_media_url);
			$group_id = $media_url->group_id;
            $channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			$c = $channels->get($channel_id);
			$title = $c->get_title();
			$show_sort[0] = '%tr%t235';
			$show_sort[1] = '%tr%t236';
			$show_sort[2] = '%tr%t237';
			$show_sort[3] = '%tr%t238';
			$ss = $show_sort[$sort_channels];
            $is_favorite = $this->tv->is_favorite_channel_id($channel_id, $plugin_cookies);
			if ($is_favorite)
				$caption = '%tr%t239';
			else
				$caption = '%tr%t240';
				$action = UserInputHandlerRegistry::create_action($this, 'add_favorite');
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $action,
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/favorites.aai');
			if ($group_id == '__all_channels'){
				if (($sort_channels == 2)||($sort_channels == 3)){
					$menu_items [] =  array(
					GuiMenuItemDef::is_separator => true,);
					$action = UserInputHandlerRegistry::create_action($this, 'up_channel');
					$menu_items[] = array(
						GuiMenuItemDef::caption => '%tr%t241',
						GuiMenuItemDef::action => $action,
						GuiMenuItemDef::icon_url => 'plugin_file://icons/ff.png');	
					$action = UserInputHandlerRegistry::create_action($this, 'lo_channel');
					$menu_items[] = array(
						GuiMenuItemDef::caption => '%tr%t242',
						GuiMenuItemDef::action => $action,
						GuiMenuItemDef::icon_url => 'plugin_file://icons/rew.png');
					$action = UserInputHandlerRegistry::create_action($this, 'first_channel');
					$menu_items[] = array(
						GuiMenuItemDef::caption => '%tr%t243',
						GuiMenuItemDef::action => $action,
						GuiMenuItemDef::icon_url => 'plugin_file://icons/up.png');
					$action = UserInputHandlerRegistry::create_action($this, 'end_channel');
					$menu_items[] = array(
						GuiMenuItemDef::caption => '%tr%t244',
						GuiMenuItemDef::action => $action,
						GuiMenuItemDef::icon_url => 'plugin_file://icons/lo.png');
					$action = UserInputHandlerRegistry::create_action($this, 'cleer_chn');
					$menu_items[] = array(
						GuiMenuItemDef::caption => '%tr%t245',
						GuiMenuItemDef::action => $action,
						GuiMenuItemDef::icon_url => 'gui_skin://special_icons/off.aai');	
				}

				$menu_items [] =  array(
						GuiMenuItemDef::is_separator => true,);
				$caption = '%tr%t246';
				$action = UserInputHandlerRegistry::create_action($this, 'search_channel');
				$menu_items[] = array(
					GuiMenuItemDef::caption => $caption,
					GuiMenuItemDef::action => $action,
					GuiMenuItemDef::icon_url => 'plugin_file://icons/search.png');

			}
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$caption = "%ext%<key_local>q__1<p>$ss</p></key_local>";
			$action = UserInputHandlerRegistry::create_action($this, 'sort_channels');
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $action,
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/apps.aai');
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$caption = '%tr%t247';
			$action = UserInputHandlerRegistry::create_action($this, 'info');
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $action,
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/info.aai');
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$parental_defs = HD::get_items('parental');
			$is_protected = (array_key_exists($channel_id, $parental_defs)) ? 1 : 0;
			if ($is_protected == 1){
				$caption = '%tr%t248';
				$action = UserInputHandlerRegistry::create_action($this, 'del_parental');
			}else{
				$caption = '%tr%t249';
				$action = UserInputHandlerRegistry::create_action($this, 'add_parental');
			}
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $action,
				GuiMenuItemDef::icon_url => 'gui_skin://special_icons/kartina_tv_lock.png'
				);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$caption = '%tr%t250';
			$action = UserInputHandlerRegistry::create_action($this, 'hide_ch');
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $action,
				GuiMenuItemDef::icon_url => 'gui_skin://special_icons/off.aai'
				);
			$caption = '%tr%t251';
			$action = UserInputHandlerRegistry::create_action($this, 'hide_list');
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $action,
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/unknown_file.aai'
				);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t252',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'change_grup'),
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/folder.aai'
				);
            $menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t254',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'select'),
				GuiMenuItemDef::icon_url => 'gui_skin://button_icons/restart.aai'
				);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t255',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'cleer_folder_key'),
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/service_file.aai'
				);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$my_channels_id = array_flip (HD::get_items('my_channels_id'));
			if (isset($my_channels_id[$channel_id]))
				$menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t256',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'del_id'),
				GuiMenuItemDef::icon_url => 'gui_skin://special_icons/off.aai'
				);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t257',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'rename'),
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/language_settings.aai'
				);
			$channels = array_flip (HD::get_items('vsetv_list'));
			if (isset($channels[$channel_id]))
				$menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t258',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'del_rename'),
				GuiMenuItemDef::icon_url => 'gui_skin://special_icons/off.aai'
				);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t222',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'add_logo'),
				GuiMenuItemDef::icon_url => 'gui_skin://small_icons/photo_viewer.aai'
				);
			if (file_exists(SmartConfig::get_data_path() . "/logo/$channel_id.png"))
			$menu_items[] = array(
                GuiMenuItemDef::caption => '%tr%t223',
                GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'del_logo'),
				GuiMenuItemDef::icon_url => 'gui_skin://special_icons/off.aai'
				);
			
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
				
			$menu_items[] = array(
            GuiMenuItemDef::caption => '%tr%t218',
            GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'add_channel_list_bg'),
			GuiMenuItemDef::icon_url => 'gui_skin://small_icons/photo_viewer.aai'
			);
			$plugin_ico = HD::get_items('plugin_ico');
			if (isset($plugin_ico['channel_list_bg'])){
				$menu_items[] = array(
				GuiMenuItemDef::caption => '%tr%t226',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'del_channel_list_bg'),
				GuiMenuItemDef::icon_url => 'gui_skin://special_icons/off.aai'
				);
			}
            return ActionFactory::show_popup_menu($menu_items);
        }
		
		if ($user_input->control_id == 'del_id')
        {
            if (!isset($user_input->selected_media_url))
                return null;
			$defs = array();
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$id = $media_url->channel_id;
			$my_channels_id = HD::get_items('my_channels_id');
			foreach ($this->tv->global_channels[$id]['caption'] as $k => $v){
				$id_key = HD::get_id_key($v);
				$add_params['id_key']= $id_key;
				if (isset($my_channels_id[$id_key]))
					ControlFactory::add_button_close ($defs, $this, $add_params,'del_id_applay',  $v, '%tr%t118', 0);
			}
            return ActionFactory::show_dialog('%tr%t259', $defs, true, 0, $attrs);
		}
		
		if ($user_input->control_id == 'del_id_applay')
        {
            if (!isset($user_input->selected_media_url))
				return null;
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$my_channels_id = HD::get_items('my_channels_id');
			unset ($my_channels_id[$user_input->id_key]);
			HD::save_items('my_channels_id', $my_channels_id);
			$id = $user_input->id_key;
			$channels_id_parsed = array_merge(
			HD::get_items('vsetv_channels_id'), 
			$my_channels_id
			);
			foreach ($this->tv->global_channels[$channel_id]['caption'] as $k => $v){
				if (HD::get_id_key($v)==$id){
					if (preg_match("|[a-zA-Z]|", $id))
						$id = array_key_exists($id,$channels_id_parsed) ? $channels_id_parsed[$id] : $id;
					$this->tv->global_channels[$id]['caption'][$k] = $this->tv->global_channels[$channel_id]['caption'][$k];
					$this->tv->global_channels[$id]['urls'][$k] = $this->tv->global_channels[$channel_id]['urls'][$k];
					unset ($this->tv->global_channels[$channel_id]['caption'][$k]);
					unset ($this->tv->global_channels[$channel_id]['urls'][$k]);
				}
			}
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
        }
		
		if ($user_input->control_id == 'del_logo')
        {
            if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
            if (file_exists(SmartConfig::get_data_path() . "/logo/$channel_id.png"))
				unlink(SmartConfig::get_data_path() . "/logo/$channel_id.png");
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
        }
		
		if ($user_input->control_id == 'add_logo')
        {
            if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
            $media_url = MediaURL::encode (
								array
								(
									'screen_id'     => 'file_list',
									'save_file'		=> array(
									'parent' 		=> 'tv_group_list',
									'action' 		=> 'ch_ico',
									'arg'			=> $media_url->channel_id,
									'gid'			=> $media_url->group_id,
									'sid'			=> self::ID,
									'extension'		=> 'png'
									),
								)
						);
			return ActionFactory::open_folder($media_url,'Folders');
        }
		
		if ($user_input->control_id === 'cleer_folder_key')
		{
			if (!isset($user_input->parent_media_url))
				return null;
			$this->tv->global_channels = null;
			$this->select_id = false;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id == 'change_grup')
		{
			if (!isset($user_input->selected_media_url))
				return null;
			if ($this->select_id==true){
				$channel_ids = $this->select_id;
				$title = '%tr%t260';
			}else{
				$media_url = MediaURL::decode($user_input->selected_media_url);
				$channel_id = $media_url->channel_id;
				$channels = $this->tv->get_channels();
				$c = $channels->get($channel_id);
				$title = $c->get_title();
			}
			
			$defs = array();
			$group_names = SmartConfig::get_group();
			$group_defs = HD::get_items('grups_id');
			if ($this->select_id!=true)
				$group_id = (array_key_exists($channel_id, $group_defs)) ? $group_defs[$channel_id] : 0;
			else
				$group_id = 0;
			foreach($group_names as $k => $v)
				$garb_ops[$k] = $v;
			ControlFactory::add_combobox($defs, $this, null,
				'group_type', '%tr%t261',
				$group_id, $garb_ops,0, $need_confirm = false, $need_apply = false
			);
			$do = UserInputHandlerRegistry::create_action($this, 'group_move');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,	'_do_garb_type', '%tr%t21', 350, $do);
			return  ActionFactory::show_dialog("%ext%<key_local>w__1<p>$title</p></key_local>",$defs,true,0,$attrs);
		}
		
		if ($user_input->control_id === 'group_move')
		{
			if (!isset($user_input->selected_media_url))
				return null;
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$group_defs = HD::get_items('grups_id');
			if ($this->select_id==true)
				$channel_ids = $this->select_id;
			else
				$channel_ids[$media_url->channel_id]=$media_url->channel_id;
			foreach ($channel_ids as $channel_id => $v)
				$group_defs[$channel_id] = $user_input->group_type;
			HD::save_items('grups_id', $group_defs);
			$this->select_id = false;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id == 'hide_list')
		{
			if (!isset($user_input->selected_media_url))
				return null;
				$hide_ch_defs = HD::get_items('hide_ch');
				if (count($hide_ch_defs) == 0)
					return ActionFactory::show_title_dialog('%tr%t262');
				$i=1;
				$cleer_hide_all = UserInputHandlerRegistry::create_action($this, 'cleer_hide_all');
				ControlFactory::add_custom_close_dialog_and_apply_buffon($defs, '_del', '%tr%t263', 350,  $cleer_hide_all);
				ControlFactory::add_vgap($defs, -25);
				ControlFactory::add_label($defs,'                                           ',' ');
				ControlFactory::add_vgap($defs, -25);
				foreach($hide_ch_defs as $key => $value){
					$add_params ['ch_pl'] = $key;
					ControlFactory::add_button_close ($defs, $this, $add_params,'cleer_hide', "$i. $value", '%tr%t264', 0);
					$i++;
				}
				$attrs['max_height'] = 740;
				return ActionFactory::show_dialog('%tr%t265', $defs, true,0,$attrs);
		}
		
		if ($user_input->control_id == 'cleer_hide_all')
		{
			if (!isset($user_input->selected_media_url))
                return null;
			HD::save_items('hide_ch', array());
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id == 'cleer_hide')
		{
			if (!isset($user_input->selected_media_url))
                return null;
			if (isset($user_input->ch_pl))
				$ch_pl = $user_input->ch_pl;
			$hide_ch_defs = HD::get_items('hide_ch');
			unset ($hide_ch_defs [$ch_pl]);
			HD::save_items('hide_ch', $hide_ch_defs);
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id == 'hide_ch')
		{
			if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
            $channels = $this->tv->get_channels();
            $c = $channels->get($channel_id);
            $title = $c->get_title();
			ControlFactory::add_label($defs, '%tr%t182', '%tr%t266');
			ControlFactory::add_label($defs, '%tr%t182', '%tr%t267');
			$do_hide_ch_apply = UserInputHandlerRegistry::create_action($this, 'hide_ch_apply');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,	'_hide_ch_apply', 'Да', 250, $do_hide_ch_apply);
			ControlFactory::add_close_dialog_button($defs,'Нет', 250);
			return  ActionFactory::show_dialog("%ext%<key_local>t__1<p>$title</p></key_local>", $defs,	true,0,$attrs);
		}
		
		if ($user_input->control_id == 'hide_ch_apply')
		{
			if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
            $channels = $this->tv->get_channels();
            $c = $channels->get($channel_id);
            $title = $c->get_title();
			$hide_ch_defs = HD::get_items('hide_ch');
			$is_hide = (array_key_exists($channel_id, $hide_ch_defs)) ? 1 : 0;
			if ($is_hide == 1)
				return ActionFactory::show_title_dialog("%ext%<key_local>y__1<p>$title</p></key_local>");
			$hide_ch_defs [$channel_id] = $title;
			HD::save_items('hide_ch', $hide_ch_defs);
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id == 'add_parental')
		{
			if (!isset($user_input->selected_media_url))
				return null;
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			$c = $channels->get($channel_id);
			$title = $c->get_title();
			$defs = array();
			ControlFactory::add_label($defs, "", '%tr%t268');
			ControlFactory::add_label($defs, "", "%ext%<key_local>u__1<p>$title</p></key_local>");
			ControlFactory::add_label($defs, "", '%tr%t269');
			ControlFactory::add_label($defs, "", '%tr%t270');
			$do_add_parental_apply = UserInputHandlerRegistry::create_action($this, 'add_parental_apply');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs, '_add_parental_apply', '%tr%t60', 250, $do_add_parental_apply);
			ControlFactory::add_close_dialog_button($defs, '%tr%t61', 250);
			return  ActionFactory::show_dialog ("%ext%<key_local>i__1<p>$title</p></key_local>", $defs, true,0,$attrs);
		}
		
		if ($user_input->control_id == 'add_parental_apply')
		{
			if (!isset($user_input->selected_media_url))
				return null;
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			$c = $channels->get($channel_id);
			$title = $c->get_title();
			$parental_defs = HD::get_items('parental');
			$is_protected = (array_key_exists($channel_id, $parental_defs)) ? 1 : 0;
			if ($is_protected == 1)
				return ActionFactory::show_title_dialog("%ext%<key_local>z__1<p>$title</p></key_local>");
			$parental_defs [$channel_id] = 1;	
			HD::save_items('parental', $parental_defs);
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id == 'del_parental')
		{
			if (!isset($user_input->selected_media_url))
				return null;
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			$c = $channels->get($channel_id);
			$title = $c->get_title();
			$parental_defs = HD::get_items('parental');
			$is_protected = (array_key_exists($channel_id, $parental_defs)) ? 1 : 0;
			if ($is_protected == 0)
				return ActionFactory::show_title_dialog("%ext%<key_local>x__1<p>$title</p></key_local>");
			$defs = array();
			ControlFactory::add_text_field($defs,0,0,
				'pin', '%tr%t271',
				'', 1, 1, 0, 1, 500, 0, false);
			$do_kill_parent_apply = UserInputHandlerRegistry::create_action($this, 'del_parent_apply');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs, '_kill_parent_apply', 'ОК', 250, $do_kill_parent_apply);
			ControlFactory::add_close_dialog_button($defs, '%tr%t9', 250);
			return  ActionFactory::show_dialog(	"%ext%<key_local>v__1<p>$title</p></key_local>",$defs,true,0,$attrs);
		}
		
		if ($user_input->control_id == 'del_parent_apply')
		{
			$pin = isset($plugin_cookies->pin) ? $plugin_cookies->pin : '0000';
			if ($user_input->pin !== $pin)
				return ActionFactory::show_title_dialog('%tr%t272');
			if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
            $channel_id = $media_url->channel_id;
            $channels = $this->tv->get_channels();
            $c = $channels->get($channel_id);
            $title = $c->get_title();
			$parental_defs = HD::get_items('parental');
			unset ($parental_defs [$channel_id]);
			HD::save_items('parental', $parental_defs);
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
        if ($user_input->control_id == 'add_favorite')
        {
            if (!isset($user_input->selected_media_url))
                return null;
			if ($this->select_id==true)
				 $channel_ids = $this->select_id;
			else{
				$media_url = MediaURL::decode($user_input->selected_media_url);
				$channel_ids[$media_url->channel_id] = $media_url->channel_id;
			}
			foreach ($channel_ids as $channel_id => $v){
				$is_favorite = $this->tv->is_favorite_channel_id($channel_id, $plugin_cookies);
				if ($is_favorite){
					$this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_REMOVE,
						$channel_id, $plugin_cookies);
					$msg = '%tr%t273';
				}
				else{
					$this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_ADD,
						$channel_id, $plugin_cookies);
					$msg = '%tr%t274';
				}
					

			}
			$this->select_id = false;
			$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
			return ActionFactory::show_title_dialog($msg,
			ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action),0,0,1);
        }
		if ($user_input->control_id == 'captions')
        {
			$do = UserInputHandlerRegistry::create_action($this, 'add_id');
			return ActionFactory::close_dialog_and_run($do);
		}
		if ($user_input->control_id == 'add_id')
        {
            $defs = array ();
			if (!isset($user_input->selected_media_url))
                return null;
            $media_url = MediaURL::decode($user_input->selected_media_url);
            $channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			if (isset($user_input->captions))
				$title = $user_input->captions;
			else{
				if (count($this->tv->global_channels[$channel_id]['caption'])==1)
					$title = current($this->tv->global_channels[$channel_id]['caption']);
				if (count($this->tv->global_channels[$channel_id]['caption'])>1){
					reset($this->tv->global_channels[$channel_id]['caption']);
					$title = current($this->tv->global_channels[$channel_id]['caption']);
					foreach($this->tv->global_channels[$channel_id]['caption'] as $k => $v)
						$captions_ops[$v] = $v;
					ControlFactory::add_combobox($defs, $this, null,
						'captions', '%tr%t275',
						$title, $captions_ops, 150, $need_confirm = false, true
					);
				}
			}
			$vsetvid = 0;
			$b = mb_substr($title,0,1,"UTF-8");
			$search = mb_strtolower($title, 'UTF-8');
			$search = str_replace(array
				(
					'нтв+',
					'(a la carte)',
					'(твоё тв)',
					'нтв плюс'
				), 	'', 
				$search);
			if (preg_match("/\s/",$search)){
				$tmp = explode(" ", $search);
				if((iconv_strlen($tmp[0], "UTF-8")) >= 3)
					$search=$tmp[0];
				else
					$search=$tmp[1];
			}
			$search = preg_quote($search);
			if (!preg_match("|[-a-zA-Z]|", $channel_id))
				$vsetvid = $channel_id;
			$ch_ops2 = array();
			$ch_ops3 = array();
			$ch_ops = array();
			$ch_ops[0] = $ch_ops2[0] = $ch_ops3[0] =
			$alfavit_ops[''] ='%tr%t116';
			$channels = array_merge(
				HD::get_items('vsetv_channels'), 
				HD::get_items('vsetv_list')
			);
			foreach($channels as $k => $v){
				$ch_ops[$v] = "$k => ". $v;
				if (preg_match("|$search|i",mb_strtolower($k, 'UTF-8')))
					$ch_ops2[$v] = "$k => ". $v;
				if (preg_match("|^$b|i", $k))
					$ch_ops3[$v] = "$k => ". $v;
			}
			$alfavit = HD::get_item('alfavit_item', &$plugin_cookies);
			$alfavit_ops = HD::alphabet();
			ControlFactory::add_label($defs, "", '%tr%t276');
			ControlFactory::add_combobox($defs, $this, null,
				'alfavit', '%tr%t277',
				$alfavit, $alfavit_ops, 150, $need_confirm = false, $need_apply = false
			);
			$do_alfavit_apply = UserInputHandlerRegistry::create_action($this, 'alfavit_vsetvid_apply');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,
			'_do_alfavit_apply', '%tr%t278', 350, $do_alfavit_apply);

			ControlFactory::add_combobox($defs, $this, null,
				'vsetvid2', '%tr%t279',
				$vsetvid, $ch_ops2, 1000, $need_confirm = false, $need_apply = false
			);
			ControlFactory::add_combobox($defs, $this, null,
				'vsetvid4', '%tr%t280',
				$vsetvid, $ch_ops3, 1000, $need_confirm = false, $need_apply = false
			);
			ControlFactory::add_combobox($defs, $this, null,
				'vsetvid3', '%tr%t281',
				$vsetvid, $ch_ops, 1000, $need_confirm = false, $need_apply = false
			);
			ControlFactory::add_text_field($defs,0,0,
				'vsetvid', '%tr%t282',
				$vsetvid, 1, 0, 0, 1, 350, 0, false);
			if (isset($user_input->captions)){
				$add_params['capt'] = $user_input->captions;
				$do_port_apply = UserInputHandlerRegistry::create_action($this, 'new_vsetvid_apply',$add_params);
			}else
				$do_port_apply = UserInputHandlerRegistry::create_action($this, 'new_vsetvid_apply');
			
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,
			'_do_vsetvid_apply', '%tr%t21', 350, $do_port_apply);
			ControlFactory::add_close_dialog_button($defs,
				'%tr%t9', 350);
			return  ActionFactory::show_dialog ("%ext%<key_local>aa__1<p>$title</p></key_local>",$defs,true, 1300,$attrs);
        }
		if ($user_input->control_id == 'alfavit_vsetvid_apply')
		{
			if (!isset($user_input->selected_media_url))
					return null;
			$alfavit = $user_input->alfavit;
			HD::save_item('alfavit_item', $alfavit);
			$defs = array();
			$ch= 0;
			$ch_ops = array();
			$ch_ops[0] = '%tr%t116';
			$channels = array_merge(
			HD::get_items('vsetv_channels'), 
			HD::get_items('vsetv_list')
			);
			foreach($channels as $k => $v){
				if (preg_match("|^$alfavit|i", $k))
					$ch_ops[$v] = "$k => ". $v;
			}
			ControlFactory::add_combobox($defs, $this, null, 'vsetvid2', '%tr%t283',
            $ch, $ch_ops, 400, $need_confirm = false, $need_apply = false
			);
			$do_port_apply = UserInputHandlerRegistry::create_action($this, 'new_vsetvid_apply');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs, '_do_vsetvid_apply', '%tr%t21', 350, $do_port_apply);
			return ActionFactory::show_dialog("%ext%<key_local>ab__1<p>$alfavit</p></key_local>",$defs, true,0,$attrs);
		}
		
		if ($user_input->control_id == 'new_vsetvid_apply')
		{
			if (!isset($user_input->selected_media_url))
				return null;
			//print_r ($user_input);
			$media_url = MediaURL::decode($user_input->selected_media_url);
			$channel_id = $media_url->channel_id;
			$channels = $this->tv->get_channels();
			$c = $channels->get($channel_id);
			if (isset($user_input->capt))
				$title = $user_input->capt;
			else
				$title = $c->get_title();
			$qb = $qb2 = $qb3 =	$qb4 = 0;
			if (isset($user_input->vsetvid))
				$qb = $user_input->vsetvid;
			if (isset($user_input->vsetvid2))
				$qb2 = $user_input->vsetvid2;
			if (isset($user_input->vsetvid3))
				$qb3 = $user_input->vsetvid3;
			if (isset($user_input->vsetvid4))
				$qb4 = $user_input->vsetvid4;
			$qq = array($qb, $qb2, $qb3, $qb4);
			foreach($qq as $key => $q)
				if ($q == intval($channel_id))
					unset($qq[$key]);
			$vsetv_id = max($qq);

			if (($vsetv_id == '') || ($vsetv_id == 0) || (!is_numeric($vsetv_id))){
				ControlFactory::add_multiline_label($defs, '', '%tr%t284', 10);
				ControlFactory::add_close_dialog_button($defs,'Ок', 350);
				return ActionFactory::show_dialog('%tr%t285', $defs,true ,1100,$attrs);
			}else
				if (isset($this->tv->global_channels[$vsetv_id]))
					$this->tv->global_channels[$vsetv_id] = array_merge_recursive($this->tv->global_channels[$vsetv_id], $this->tv->global_channels[$channel_id]);
				else
					$this->tv->global_channels[$vsetv_id] = $this->tv->global_channels[$channel_id];
				unset ($this->tv->global_channels[$channel_id]);
				$id_key = HD::get_id_key($title);
				$my_channels_id = HD::get_items('my_channels_id');
				$my_channels_id [$id_key] = $vsetv_id;
				HD::save_items('my_channels_id', $my_channels_id);
				if ($vsetv_id > 2999){
					$texts = HD::get_items('vsetv_list');
					$key = array_search($vsetv_id, $texts);
					if ($key == false){
						$texts [$title] = $vsetv_id;
						HD::save_items('vsetv_list', $texts);
					}
					if ((!file_exists("/D/$vsetv_id.png")) && (!file_exists(SmartConfig::get_data_path()  . "/logo/$vsetv_id.png"))){
						$media_url = MediaURL::encode (
								array
								(
									'screen_id'     => 'file_list',
									'save_file'		=> array(	
									'parent' 		=> 'tv_group_list',
									'action' 		=> 'ch_ico',
									'arg'			=> $vsetv_id,
									'gid'			=> $media_url->group_id,
									'sid'			=> self::ID,
									'extension'		=> 'png'
									),
								)
						);
						return ActionFactory::open_folder($media_url,'Folders');
					}
					if (file_exists("/D/$vsetv_id.png"))
						shell_exec("cp -f /D/$vsetv_id.png " . SmartConfig::get_data_path() . "/logo/$vsetv_id.png");
				}
				$perform_new_action = UserInputHandlerRegistry::create_action($this, 'cleer_folder');
				return ActionFactory::invalidate_folders(array('tv_group_list'), $perform_new_action);
		}
		
		if ($user_input->control_id === 'cleer_folder')
		{
			if (!isset($user_input->parent_media_url))
				return null;
			$parent_media_url = MediaURL::decode($user_input->parent_media_url);
			$sel_ndx = $user_input->sel_ndx;
			if (isset($user_input->cursor)){
				if ($user_input->cursor == 0)
					$sel_ndx = 0;
				else
					$sel_ndx = $sel_ndx + $user_input->cursor;
			}
			if ($sel_ndx < 0)
				$sel_ndx = 0;
			$range = $this->get_folder_range($parent_media_url, 0, $plugin_cookies);
			return ActionFactory::update_regular_folder($range, true, $sel_ndx);
		}
		if ($user_input->control_id === 'cleer_folder_plus')
		{
			if (!isset($user_input->parent_media_url))
				return null;
			$parent_media_url = MediaURL::decode($user_input->parent_media_url);
			$sel_ndx = $user_input->sel_ndx+1;
				if ($sel_ndx < 0)
					$sel_ndx = 0;
			$range = $this->get_folder_range($parent_media_url, 0, $plugin_cookies);
			return ActionFactory::update_regular_folder($range, true, $sel_ndx);
		}
        return null;
    }

    ///////////////////////////////////////////////////////////////////////

    private function get_regular_folder_item($group, $c, &$plugin_cookies)
    {
		$color = 15;
		if (isset($this->select_id[$c->get_id()]))
			$color = 18;
        return array
        (
            PluginRegularFolderItem::media_url =>
                MediaURL::encode(
                    array(
                        'channel_id' => $c->get_id(),
                        'group_id' => $group->get_id())),
            PluginRegularFolderItem::caption => $c->get_title(),
            PluginRegularFolderItem::view_item_params => array
            (
                ViewItemParams::icon_path => $c->get_icon_url(),
                ViewItemParams::item_detailed_icon_path => $c->get_icon_url(),
				ViewItemParams::item_caption_color => $color,
            ),
            PluginRegularFolderItem::starred =>
                $this->tv->is_favorite_channel_id(
                    $c->get_id(), $plugin_cookies),
        );
    }

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $this->tv->folder_entered($media_url, $plugin_cookies);

        //$this->tv->ensure_channels_loaded($plugin_cookies);

        $group = $this->tv->get_group($media_url->group_id);

        $items = array();

        foreach ($group->get_channels($plugin_cookies) as $c)
        {
            $items[] = $this->get_regular_folder_item(
                $group, $c, $plugin_cookies);
        }

        return $items;
    }

    public function get_archive(MediaURL $media_url)
    {
        return $this->tv->get_archive($media_url);
    }
}

///////////////////////////////////////////////////////////////////////////
?>
